@@ -56,7 +56,7 @@ static char ripe_server[] = "whois.ripe.net";
 static char ripe_ris_server[] = "riswhois.ripe.net";
 
 #ifdef STANDALONE
-const char *version = "3.76";                    /* set version string for library and client */
+const char *version = "3.77";                   /* set version string for library and client */
 const char *appname = "WhoB";            	    /* set application name */
 static int go_interactive = 0;                  /* We'll wait on STDIN unless args suggest otherwise */
 static int use_cymru = 0;                       /* Don't use Cymru by default */
@@ -66,6 +66,7 @@ static int display_aspath = 0;                  /* Don't display AS-PATH by defa
 static int display_netname = 0;                 /* Don't display netname by default */
 static int display_radb_as = 0;                 /* Don't display RADB Origin-AS by default */
 static int show_routes_byasn = 0;               /* Don't show all advertisements by default */
+static int show_routes_bytransitasn = 0;        /* Don't show all routes that transit ASN by default */
 static int show_networks_byasn = 0;             /* Don't show all networks by default */
 static int show_contacts_byasn = 0;             /* Don't show all contact info by default */
 static int show_routes_byprefix = 0;            /* Don't show all routes by prefix by default */
@@ -331,15 +332,47 @@ static char *match_inparens(char *target)
 
 static char *match_afterparens(char *target)
 {
-    /* matches something like "   (HELLO) xxx" and returns a pointer to "xxx" */
-    target = strchr(target, '(');
-    if (!target) return NULL;
-    target = strchr(target, ')');
-    if (!target) return NULL;
-    target++;
-    while(*target && isspace(*target)) target++;
-    if (*target) return strdup(target);
-    else return NULL;
+	/* matches something like "   (HELLO) xxx" and returns a pointer to "xxx" */
+	target = strchr(target, '(');
+	if (!target) return NULL;
+	target = strchr(target, ')');
+	if (!target) return NULL;
+	target++;
+	while(*target && isspace(*target)) target++;
+	if (*target) return strdup(target);
+	else return NULL;
+}
+
+static int stricontains(const char *haystack, const char *needle) 
+{
+	/* Search for a substring without a string library */
+	int i, j, match;
+	i = 0, j = 0;
+	
+	while (haystack[i] != '\0') {
+	
+	  while (tolower(haystack[i]) != tolower(needle[0]) && haystack[i] != '\0')
+		 i++;
+	
+	  if (haystack[i] == '\0')
+		 return (-1);
+	
+	  match = i;
+	
+	  while (tolower(haystack[i]) == tolower(needle[j]) && haystack[i] != '\0' && needle[j] != '\0') {
+		 i++;
+		 j++;
+	  }
+	
+	  if (needle[j] == '\0')
+		 return (match);
+	  if (haystack[i] == '\0')
+		 return (-1);
+	
+	  i = match + 1;
+	  j = 0;
+	}
+	return 0;
 }
 
 whois_session_params *w_init(void)
@@ -1032,6 +1065,48 @@ static int w_display_rvbyasn_pwhois(whois_session_params * wsess, char *asn)
     return 0;
 }
 
+static int w_display_rvbytransitasn_pwhois(whois_session_params * wsess, char *asn)
+{
+    char *reply;
+    const char *query_begin = "routeview transit-as=";
+    char *whob_query = NULL;
+    char *serv;
+    
+    if (!asn) 
+        return -1;
+    
+    if (strlen(wsess->pw_serv) > 0)
+        serv = wsess->pw_serv;
+    else
+        serv = pwhois_server;
+    
+    whob_query = malloc( (strlen(appname) + strlen(version) + strlen(query_begin) + strlen(asn)) * sizeof(char) + 10);
+    whob_query[0]=0;
+    
+    /* prepare the text-string-based query */
+    strcat(whob_query,"app=\"");
+    strcat(whob_query,appname);
+    strcat(whob_query," ");
+    strcat(whob_query,version);
+    strcat(whob_query,"\" ");
+    strcat(whob_query,query_begin);
+    strcat(whob_query,asn);
+    
+    strcat(whob_query,"\n");
+    
+    reply = w_ask(serv, whob_query, NULL);
+    if (!reply) {
+        if (wsess->w_noisy) fprintf(stderr,"No reply from %s.\n",serv);
+        return -1;
+    }
+    
+    printf("%s",reply);
+    
+    free(reply); free(whob_query);
+    
+    return 0;
+}
+
 static int w_display_contactsbyasn_pwhois(whois_session_params * wsess, char *asn)
 {
     char *reply;
@@ -2260,6 +2335,7 @@ usage (char *prog)
              "  -c         Use Cymru\'s whois server instead of pWhoIs\n"
              "  -r         Use RIPE NCC\'s RIS whois server instead of pWhoIs\n"
              "\nPrefix WhoIs Advanced Options:\n"
+             "  -A         Display all routes transiting the target ASN (pWhoIs-only)\n"
              "  -a         Display all routes announced by the target ASN (pWhoIs-only)\n"
              "  -P         Display all routes respective to a target prefix (pWhoIs-only)\n"
              "  -N         Display all networks registered to the target ASN (pWhoIs-only)\n"
@@ -2295,7 +2371,7 @@ show_startup_msgs (whois_session_params * wsess)
                 else fprintf(stderr,"Using Prefix WhoIs for bulk file resolution.\n");
             }
         }
-        if (show_routes_byprefix == 1 && show_routes_byasn == 1) {
+        if (show_routes_byprefix == 1 && (show_routes_byasn == 1 || show_routes_bytransitasn == 1)) {
             fprintf(stderr,"You may only perform routeviews one at a time.  Using by-ASN.\n");
             show_routes_byprefix = 0;
         } 
@@ -2317,13 +2393,18 @@ int main(int ac, char **av)
     wsess = w_init();
     setbuf(stdout, NULL);
         
-    while ((ch = getopt (ac, av, "aCcfgNnOopPRrstuVvh:w:")) != EOF)
+    while ((ch = getopt (ac, av, "AaCcfgNnOopPRrstuVvh:w:")) != EOF)
         switch (ch) {
             case 'a':
                 use_gigo = 0;
                 show_routes_byasn = 1;
                 go_interactive = 1;
                 break;
+            case 'A':
+                use_gigo = 0;
+                show_routes_bytransitasn = 1;
+                go_interactive = 1;
+                break;
             case 'N':
                 use_gigo = 0;
                 show_networks_byasn = 1;
@@ -2511,7 +2592,7 @@ int main(int ac, char **av)
         exit(EXIT_SUCCESS);
     }
     
-    if ((show_routes_byasn || show_contacts_byasn || show_networks_byasn) && (strlen(hostname) <= 10) && atoi(hostname)) {
+    if ((show_routes_byasn || show_routes_bytransitasn || show_contacts_byasn || show_networks_byasn) && (strlen(hostname) <= 10) && atoi(hostname) && stricontains(hostname,".") < 1) {
         user_asn = atoi(hostname);
         if (wsess->w_noisy > 1) fprintf(stderr,"Using user-supplied ASN %d for lookup.\n",user_asn);
     } else if (show_routes_byprefix == 1) {
@@ -2544,6 +2625,20 @@ int main(int ac, char **av)
                 printf("Sorry, unable to resolve the ASN for %s (%s) at this time.\n",hostname,addr);
             }
         }
+    } else if (show_routes_bytransitasn) {
+        if (user_asn > 0) {
+            printf("Displaying all routes whose Transit-AS is %d.\n", user_asn);
+            snprintf(user_asn_buf,9,"%d",user_asn);
+            w_display_rvbytransitasn_pwhois(wsess, user_asn_buf);
+        } else {
+            w_lookup_all_pwhois(wsess, addr);
+            if (atoi(wsess->consolidated_asn)) {
+                printf("Displaying all routes whose Transit-AS is %s.\n", wsess->consolidated_asn);
+                w_display_rvbytransitasn_pwhois(wsess, wsess->consolidated_asn);
+            } else {
+                printf("Sorry, unable to resolve the ASN for %s (%s) at this time.\n",hostname,addr);
+            }
+        }
     } else if (show_networks_byasn) {
         if (user_asn > 0) {
             printf("Displaying all networks registered to the Origin-AS %d.\n", user_asn);
